﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using XNA4;

namespace XNA4Model
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Set the 3D model to draw.
        ModelNode[] modelList = new ModelNode[4];

        // Set the position of the model in world space, and set the rotation.
        Vector3 modelPosition = Vector3.Zero;
        float modelRotation = 0.0f;
        float modelRotationPitch = 0.0f;
        // Set the velocity of the model, applied each frame to the model's position.
        Vector3 modelVelocity = Vector3.Zero;
        // Set the position of the camera in world space, for our view matrix.
        Vector3 cameraPosition = new Vector3(0.0f, 0.0f, 1000.0f);
        // The aspect ratio determines how to scale 3d to 2d projection.
        float aspectRatio;
        Matrix world;
        Matrix view;
        Matrix projection;

        /// <summary>
        /// input
        /// </summary>
        static InputHandler inputs;
        /// <summary>
        /// FPS Counter
        /// </summary>
        static FpsCounter fpsCounter;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            /// <summary>
            /// inputs component
            /// </summary>
            inputs = new InputHandler(this);
            Components.Add(inputs);
            /// <summary>
            /// fps component
            /// </summary>
            fpsCounter = new FpsCounter(this);
            Components.Add(fpsCounter);

            world = Matrix.CreateRotationY(modelRotation) * Matrix.CreateRotationX(modelRotationPitch) * Matrix.CreateTranslation(modelPosition);
            view = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 10000.0f);

            modelList[0] = new ModelNode(this, "Model\\p1_wedge", Vector3.Zero, new Vector3(0.2f, 0.2f, 0.2f), Quaternion.CreateFromAxisAngle(new Vector3(0, 1, 0), MathHelper.Pi));
            modelList[0].World = world;
            modelList[0].View = view;
            modelList[0].Projection = projection;
            Components.Add(modelList[0]);
            modelList[1] = new ModelNode(this, "Model\\airplane2", new Vector3(-400, 0, 0), new Vector3(70f, 70f, 70f), Quaternion.Identity);
            modelList[1].World = world;
            modelList[1].View = view;
            modelList[1].Projection = projection;
            Components.Add(modelList[1]);
            modelList[2] = new ModelNode(this, "Model\\cactus", new Vector3(400, 0, 0), new Vector3(1f, 1f, 1f), Quaternion.Identity);
            modelList[2].World = world;
            modelList[2].View = view;
            modelList[2].Projection = projection;
            Components.Add(modelList[2]);
            modelList[3] = new ModelNode(this, "Model\\starfish", new Vector3(0, 300, 0), new Vector3(10f, 10f, 10f), Quaternion.Identity);
            modelList[3].World = world;
            modelList[3].View = view;
            modelList[3].Projection = projection;
            Components.Add(modelList[3]);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            // Get some input.
            UpdateInput();

            // Add velocity to the current position.
            modelPosition += modelVelocity;
            // Bleed off velocity over time.
            modelVelocity *= 0.95f;

            Matrix world = Matrix.CreateRotationY(modelRotation) * Matrix.CreateRotationX(modelRotationPitch) * Matrix.CreateTranslation(modelPosition);
            foreach (ModelNode model in modelList)
                model.World = world;

            base.Update(gameTime);
        }

        protected void UpdateInput()
        {
            float speed = 50.0f;
            if (inputs.PauseGame)
            {
                this.Exit();
            }
            if (inputs.IsKeyPressed(Keys.Up))
            {
                modelPosition += Vector3.Up * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Down))
            {
                modelPosition += Vector3.Down * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Right))
            {
                modelPosition += Vector3.Right * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Left))
            {
                modelPosition += Vector3.Left * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Space))
            {
                modelPosition = Vector3.Zero;
                modelVelocity = Vector3.Zero;
                modelRotation = 0.0f;
                modelRotationPitch = 0.0f;
            }
            if (inputs.IsKeyDown(Keys.A))
            {
                modelRotation += 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyDown(Keys.D))
            {
                modelRotation -= 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyDown(Keys.W))
            {
                modelRotationPitch += 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyDown(Keys.S))
            {
                modelRotationPitch -= 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            //modelRotation -= inputs.MouseMoved.X * 0.1f;
            //modelRotationPitch -= inputs.MouseMoved.Y * 0.1f;

            // Get the game pad state.
            GamePadState currentState = inputs.CurrentGamePadState;
            if (currentState.IsConnected)
            {
                // Rotate the model using the left thumbstick, and scale it down.
                modelRotation -= currentState.ThumbSticks.Left.X * 0.10f;

                // Create some velocity if the right trigger is down.
                Vector3 modelVelocityAdd = Vector3.Zero;

                // Find out what direction we should be thrusting, using rotation.
                modelVelocityAdd.X = -(float)Math.Sin(modelRotation);
                modelVelocityAdd.Z = -(float)Math.Cos(modelRotation);

                // Now scale our direction by how hard the trigger is down.
                modelVelocityAdd *= currentState.Triggers.Right;

                // Finally, add this vector to our velocity.
                modelVelocity += modelVelocityAdd;

                GamePad.SetVibration(PlayerIndex.One, currentState.Triggers.Right,
                    currentState.Triggers.Right);

                // In case you get lost, press A to warp back to the center.
                if (currentState.Buttons.A == ButtonState.Pressed)
                {
                    modelPosition = Vector3.Zero;
                    modelVelocity = Vector3.Zero;
                    modelRotation = 0.0f;
                }
            }
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
